#!/usr/bin/python
# encoding: utf-8
#
# script utilizado para converter os dados de coautoria (obtidos pelo scriptLattes) em arquivos
# úteis para análise por ferramentas externas:
# - Arquivo GML: Rede de coautoria em formato Graph Markup Language (uso no software R-project)
# - Arquivo GDF: Rede de coautoria em formato Gephi (uso no software Gephi)
#
# URL: http://scriptlattesblog.blogspot.com
#
# Copyright - Jesús P. Mena-Chalco <jesus.mena@ufabc.edu.br>
# Dom Mai 12 23:42:33 BRT 2013
# Sex Jul 19 00:12:09 BRT 2013
#

import sys
import shutil
import os, errno
import pygraphviz
import fileinput
import numpy

def fileLen(fmatriz):
	with open(fmatriz) as f:
		for i, l in enumerate(f):
			pass
	return i + 1

def salvarArquivo(conteudo, arquivo):
	arquivo = open(arquivo, 'w')
	arquivo.write(conteudo)
	arquivo.close()

# --------------------------------------------------------------------------------- #
# Programa principal
# --------------------------------------------------------------------------------- #
if __name__ == "__main__":
	fMatriz  = sys.argv[1]
	fNomes   = sys.argv[2]
	fRotulos = sys.argv[3]
	prefixo  = sys.argv[4] 
	opcao    = "-nd" 
	foutGML  = prefixo+".gml"
	foutGDF  = prefixo+".gdf"

	N = fileLen(fMatriz)
	matriz = numpy.zeros((N,N), dtype=numpy.float32)

	# ---------------------------------------------------------------------------- #
	# Leitura da matriz de adjacência
	# ---------------------------------------------------------------------------- #
	with open(fMatriz) as f:
		for i, linha in enumerate(f):
			vetor = linha.split()
			for j in range(0,N):
				matriz[i][j] = vetor[j]

	# ---------------------------------------------------------------------------- #
	# Leitura dos nomes
	# ---------------------------------------------------------------------------- #
	nomes = []
	for linha in fileinput.input(fNomes):
		linha = linha.replace("\r","")
		linha = linha.replace("\n","")
		nomes.append(linha.strip())

	# ---------------------------------------------------------------------------- #
	# Leitura dos rotulos
	# ---------------------------------------------------------------------------- #
	rotulos = []
	for linha in fileinput.input(fRotulos):
		linha = linha.replace("\r","")
		linha = linha.replace("\n","")
		rotulos.append(linha.strip())

	rotulosGrupos = list(set(rotulos))
	NG = len(rotulosGrupos)
	matrizGrupos = numpy.zeros((NG,NG), dtype=numpy.float32)

	rotulosIndices = []
	# para cada pesquisador
	for i in range(0,N):
		for indice in range(0,len(rotulosGrupos)):
			if rotulosGrupos[indice]==rotulos[i]:
				rotulosIndices.append(indice)
				break

	#print matriz
	#print rotulosIndices
	#print rotulos
	#print sum(sum(matriz))
				
	# Arestas
	for i in range(0,N):
		for j in range(i,N):
			if (i!=j) and (matriz[i][j]>0):
	 			grupoI = rotulosIndices[i]
				grupoJ = rotulosIndices[j]
				matrizGrupos[grupoI][grupoJ] += matriz[i][j]
				matrizGrupos[grupoJ][grupoI] += matriz[i][j]

	#print matrizGrupos

	# Dados a serem considerados
	# --> rotulosGrupos
	# --> matrizGrupos

	# ---------------------------------------------------------------------------- #
	# ---------------------------------------------------------------------------- #
	# ---------------------------------------------------------------------------- #
	# ---------------------------------------------------------------------------- #
	# ---------------------------------------------------------------------------- #
	# Geração do XML em formato graphml
	# http://graphml.graphdrawing.org/primer/graphml-primer.html
	# http://igraph.sourceforge.net/screenshots2.html#5
	# ---------------------------------------------------------------------------- #
	
	string = '<?xml version="1.0" encoding="UTF-8"?> \
\n<graphml xmlns="http://graphml.graphdrawing.org/xmlns" \
\n  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" \
\n  xsi:schemaLocation="http://graphml.graphdrawing.org/xmlns \
\n  http://graphml.graphdrawing.org/xmlns/1.0/graphml.xsd">'

	# Tipos de atributos (data schema)
	string += '\n<key id="name" for="node" attr.name="name" attr.type="string"/>'
	string += '\n<key id="label" for="node" attr.name="label" attr.type="string"/>'
	string += '\n<key id="weight" for="edge" attr.name="weight" attr.type="double"/>'

	if opcao=='-d':
		string += '\n<graph id="G" edgedefault="directed">'
	else:
		string += '\n<graph id="G" edgedefault="undirected">'

	# Vêrtices
	for i in range(0,NG):
		string +='\n<node id="'+str(i)+'">'
		string +='\n  <data key="name">'+rotulosGrupos[i]+'</data>'
		string +='\n  <data key="label">'+rotulosGrupos[i]+'</data>'
		string +='\n</node>'

	# Arestas
	for i in range(0,NG):
		for j in range(i,NG):
			#if (i!=j) and (matrizGrupos[i][j]>0):
			if (matrizGrupos[i][j]>0):
				string +='\n<edge source="'+str(i)+'" target="'+str(j)+'">'
				string +='\n  <data key="weight">'+str(matrizGrupos[i][j])+'</data>'
				string +='\n</edge>'
	string+='\n</graph>'
	string+='\n</graphml>'

	salvarArquivo(string, foutGML)

	# ---------------------------------------------------------------------------- #
	# Criacao de arquivo GDF (útil para o Gephi)                                   #
	# ---------------------------------------------------------------------------- #
	arquivo       = string.split("\n")
	string        = ""
	inNode        = False
	comecaArestas = False

	# ---------------------------------------------------------------------------- #
	string = "nodedef> name VARCHAR, label VARCHAR, grupo VARCHAR\n"
	estamosDentroDeAresta = False

	for line in arquivo:
		line = line.strip()
		if (estamosDentroDeAresta):
			wwt = line
			wwt = wwt.replace('<data key="weight">','')
			wwt = wwt.replace('</data>','')
			wwt = wwt.strip()
			string += " , " + wwt + " \n"
			estamosDentroDeAresta = False
		else: 
			if (line == "</node>"):
				inNode = False
				string += idNode + " , " + idName + " , " + idLabel + "\n"
			if ("<node id=" in line):
				idNode = line.lstrip('<node id=').strip('>')
				idNode = idNode.strip('"')
				inNode = True
			if (inNode):
				if ('<data key="name">' in line):
					idName = line
					idName = idName.replace('<data key="name">','')
					idName = idName.replace('</data>','')
				if ('<data key="label">' in line):
					idLabel = line
					idLabel = idLabel.replace('<data key="label">','')
					idLabel = idLabel.replace('</data>','')
			# -------------------------------------------------------------------- #
			if ('<edge source=' in line and not comecaArestas):
				comecaArestas = True
				string += "edgedef> node1 VARCHAR, node2 VARCHAR, weight DOUBLE\n"
			if ("<edge source=" in line):
				line = line.replace('<edge source="','')
				line = line.replace('" target="',' , ')
				line = line.replace('">','')
				string += line	
				estamosDentroDeAresta = True

	# ---------------------------------------------------------------------------- #
	string = string.strip('\n')
	salvarArquivo(string, foutGDF)

	# ---------------------------------------------------------------------------- #
	# Fim                                                                          #
	# ---------------------------------------------------------------------------- #
